@@ -18,12 +18,13 @@ sub build_bundle {
 	my $bundle_contents = "SDLPerl.app/Contents";
 	system "mkdir -p \"$bundle_contents\"";
 	mkdir "$bundle_contents/MacOS", 0755;
+	my $Perl = ($ENV{'FULLPERL'} or $^X or 'perl') ;
 	my $cflags; #= Alien::SDL->config('cflags');
-	$cflags = ' ' . `$^X -MExtUtils::Embed -e ccopts`;
+	$cflags = ' ' . `$Perl -MExtUtils::Embed -e ccopts`;
 	chomp($cflags);
 	$cflags .= ' ' . Alien::SDL->config('cflags');
 	my $libs;   #= Alien::SDL->config('libs');
-	$libs = ' ' . `$^X -MExtUtils::Embed -e ldopts`;
+	$libs = ' ' . `$Perl -MExtUtils::Embed -e ldopts`;
 	chomp($libs);
 	$libs .= ' ' . Alien::SDL->config('libs') . ' -lSDLmain';
 	chomp($libs);
